package com.taauav.common.util;

import net.coobird.thumbnailator.Thumbnails;
import net.coobird.thumbnailator.name.Rename;

import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * <p>图片压缩工具类</p>
 *
 * @author : dyg
 * @date : 2019-11-25 08:57
 **/
public class ImageUtil {

    //图片默认压缩比例
    private static final double DEFAULT_SCALE = 0.8d;

    //缩略图后缀
    private static final String SUFFIX = "-thumbnail";

    /**
     * 生成缩略图到指定目录
     * @param path
     * @param files
     * @return
     * @throws IOException
     */
    public static List<String> generateThumbnail2Directory(String path, String... files) throws IOException {
        return generateThumbnail2Directory(DEFAULT_SCALE,path,files);
    }

    /**
     * 生成缩略图到指定路径
     * @param scale
     * @param pathname
     * @param files
     * @return
     * @throws IOException
     */
    public static List<String> generateThumbnail2Directory(double scale,String pathname, String... files) throws IOException {
        Thumbnails.of(files).scale(scale).toFiles(new File(pathname), Rename.SUFFIX_HYPHEN_THUMBNAIL);
        List<String> list = new ArrayList<>(files.length);
        for (String file : files) {
            list.add(appendSuffix(file,SUFFIX));
        }
        return list;
    }

    /**
     * 将指定目录下的所有图片生成缩略图
     * @param pathname
     * @throws IOException
     */
    public static void generateDirectoryThumbnail(String pathname) throws IOException {
        generateDirectoryThumbnail(pathname, DEFAULT_SCALE);
    }

    /**
     * 将指定目录下所有图片生成缩略图
     * @param pathname
     * @param scale
     * @throws IOException
     */
    public static void generateDirectoryThumbnail(String pathname, double scale) throws IOException {
        File[] files = new File(pathname).listFiles();
        compressRecurse(files, pathname);
    }

    /**
     * <p>Title: thumbnailImage</p>
     * <p>Description: 根据图片路径生成缩略图 </p>
     * @param imgFile    原图片路径
     * @param w            缩略图宽
     * @param h            缩略图高
     * @param destFile    生成缩略图的地址
     * @param force        是否强制按照宽高生成缩略图(如果为false，则生成最佳比例缩略图)
     */
    public static String thumbnailImage(File imgFile, int w, int h, String destFile, boolean force){
        String str = "";
        if(imgFile.exists()){
            try {
                // ImageIO 支持的图片类型 : [BMP, bmp, jpg, JPG, wbmp, jpeg, png, PNG, JPEG, WBMP, GIF, gif]
                String types = Arrays.toString(ImageIO.getReaderFormatNames());
                String suffix = null;
                // 获取图片后缀
                if(imgFile.getName().indexOf(".") > -1) {
                    suffix = imgFile.getName().substring(imgFile.getName().lastIndexOf(".") + 1);
                }// 类型和图片后缀全部小写，然后判断后缀是否合法
                if(suffix == null || types.toLowerCase().indexOf(suffix.toLowerCase()) < 0){
                    return "";
                }
                Image img = ImageIO.read(imgFile);
                if(!force){
                    // 根据原图与要求的缩略图比例，找到最合适的缩略图比例
                    int width = img.getWidth(null);
                    int height = img.getHeight(null);
                    if((width*1.0)/w < (height*1.0)/h){
                        if(width > w){
                            h = Integer.parseInt(new java.text.DecimalFormat("0").format(height * w/(width*1.0)));
                        }
                    } else {
                        if(height > h){
                            w = Integer.parseInt(new java.text.DecimalFormat("0").format(width * h/(height*1.0)));
                        }
                    }
                }
                BufferedImage bi = new BufferedImage(w, h, BufferedImage.TYPE_INT_RGB);
                Graphics g = bi.getGraphics();
                g.drawImage(img, 0, 0, w, h, Color.LIGHT_GRAY, null);
                g.dispose();
                String p = imgFile.getPath();
                // 将图片保存在原目录并加上前缀
                ImageIO.write(bi, suffix, new File(destFile));
                str = destFile;
            } catch (IOException e) {
                e.printStackTrace();

            }
        }
        return str;
    }

    /**
     * 文件追加后缀
     * @param fileName 原文件名
     * @param suffix   后缀
     * @return
     */
    public static String appendSuffix(String fileName, String suffix) {
        String newFileName = "";
        int indexOfDot = fileName.lastIndexOf('.');
        if (-1 != indexOfDot) {
            newFileName = fileName.substring(0,indexOfDot);
            newFileName += suffix;
            newFileName += fileName.substring(indexOfDot);
        } else {
            newFileName = fileName + suffix;
        }
        return newFileName;
    }

    private static void compressRecurse(File[] files, String pathname) throws IOException {
        for (File file : files) {
            if (file.isDirectory()) {
                File[] subFiles = file.listFiles();
                compressRecurse(subFiles, pathname + File.separator+file.getName());
            } else {
                String extension = getFileExtension(file.getName());
                if (!file.getName().contains(SUFFIX) && isImage(extension)) {
                    generateThumbnail2Directory(pathname, file.getAbsolutePath());
                }
            }
        }
    }

    /**
     * 根据文件扩展名判断文件是否为图片格式
     * @param extension
     * @return
     */
    public static boolean isImage(String extension) {
        String[] imageExtension = new String[]{"jpeg","jpg","gif","bmp","png"};
        for (String e: imageExtension) {
            if (extension.toLowerCase().equals(e)) {
                return true;
            }
        }
        return false;
    }
    /**
     * 根据文件获取后缀名
     * @param fileName
     * @return
     */
    public static String getFileExtension(String fileName) {
        String extension = fileName.substring(fileName.lastIndexOf('.') + 1);
        return extension;
    }
}
